/*
 * InstructionObjectStatus.java
 */

package com.sierra.agi.logic.interpret.instruction;

import com.sierra.agi.*;
import com.sierra.agi.logic.*;
import com.sierra.agi.logic.interpret.*;
import java.io.*;

/**
 * Object Status Instruction.
 *
 * <P><CODE><B>object.status</B> Instruction 0x85</CODE><BR>
 * Prints a message for the object <CODE>o[v[p1]]</CODE> in the format
 * <PRE>
 * Obj <n> x: <pos> y: <pos> pri: <priority> stepsize: <step size>.
 * </PRE>
 * </P>
 *
 * @author  Dr. Z
 * @version 0.00.00.01
 */
public class InstructionObjectStatus extends InstructionUni
{
    /** 
     * Creates new Object Status Instruction.
     *
     * @param context   Game context where this instance of the instruction will be used. (ignored)
     * @param stream    Logic Stream. Instruction must be written in uninterpreted format.
     * @param reader    LogicReader used in the reading of this instruction. (ignored)
     * @param bytecode  Bytecode of the current instruction.
     * @throws IOException I/O Exception are throw when <CODE>stream.read()</CODE> fails.
     */
    public InstructionObjectStatus(Context context,InputStream stream,LogicReader reader,short bytecode) throws IOException
    {
        super(context, stream, reader, bytecode);
    }
    
    /**
     * Execute the Instruction.
     *
     * @param logic         Logic used to execute the instruction.
     * @param logicContext  Logic Context used to execute the instruction.
     * @return Returns the number of byte of the uninterpreted instruction.
     */
    public int execute(Logic logic, LogicContext logicContext)
    {
        return 2;
    }

//#ifdef DEBUG
    /**
     * Retreive the AGI Instruction name and parameters.
     * <B>For debugging purpose only. Will be removed in final releases.</B>
     *
     * @return Returns the textual names of the instruction.
     */
    public String[] getNames()
    {
        return new String[] {"obj.status", "ov" + p1};
    }
//#endif DEBUG
}