/*
 * InstructionStepSize.java
 */

package com.sierra.agi.logic.interpret.instruction;

import com.sierra.agi.*;
import com.sierra.agi.logic.*;
import com.sierra.agi.logic.interpret.*;
import java.io.*;

/**
 * Step Size Instruction.
 *
 * <P><CODE><B>step.size</B> Instruction 0x4F</CODE><BR>
 * <CODE>v[p2]</CODE> determines the number of pixels the object <CODE>o[p1]</CODE>
 * moves each step. <I>(The actual value in pixels is the step size / 4)</I>
 * </P>
 *
 * @author  Dr. Z
 * @version 0.00.00.01
 */
public class InstructionStepSize extends InstructionBi
{
    /** 
     * Creates new Step Size Instruction.
     *
     * @param context   Game context where this instance of the instruction will be used. (ignored)
     * @param stream    Logic Stream. Instruction must be written in uninterpreted format.
     * @param reader    LogicReader used in the reading of this instruction. (ignored)
     * @param bytecode  Bytecode of the current instruction.
     * @throws IOException I/O Exception are throw when <CODE>stream.read()</CODE> fails.
     */
    public InstructionStepSize(Context context,InputStream stream,LogicReader reader,short bytecode) throws IOException
    {
        super(context, stream, reader, bytecode);
    }
    
    /**
     * Execute the Instruction.
     *
     * @param logic         Logic used to execute the instruction.
     * @param logicContext  Logic Context used to execute the instruction.
     * @return Returns the number of byte of the uninterpreted instruction.
     */
    public int execute(Logic logic, LogicContext logicContext)
    {
        return 3;
    }

//#ifdef DEBUG
    /**
     * Retreive the AGI Instruction name and parameters.
     * <B>For debugging purpose only. Will be removed in final releases.</B>
     *
     * @return Returns the textual names of the instruction.
     */
    public String[] getNames()
    {
        return new String[] {"step.size", "o" + p1, "v" + p2};
    }
//#endif DEBUG
}